package com.xhkjedu.sexam.controller.exam;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.sexam.model.exam.EBase;
import com.xhkjedu.sexam.model.exam.EClassStudent;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.service.exam.EBaseService;
import com.xhkjedu.sexam.vo.exam.ExamVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname EBaseController
 * @description 考试基础信息
 * @date 2022/7/18 9:26
 **/
@RestController
@RequestMapping("/eb")
public class EBaseController {
    @Resource
    private EBaseService eBaseService;

    /**
     * 发起考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 10:15
     **/
    @PostMapping("/add")
    public ResultVo add(@RequestBody EBase eBase) {
        if (N_Utils.isListEmpty(eBase.getClassids())) return new ResultVo(1, "考试班级不能为空");
        if (N_Utils.isListEmpty(eBase.getSubjectids())) return new ResultVo(1, "考试科目不能为空");
        return eBaseService.save(eBase);
    }

    /**
     * 考试设置
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [subjects]
     * @Author ywx
     * @Date 2022/7/18 10:52
     **/
    @PostMapping("/set_exam")
    public ResultVo setExam(@RequestBody ExamVo map) {
        Integer notice = map.getNotice();//是否发送通知1发送2不发送
        Integer msg = map.getMsg();//是否发送短信1发送2不发送
        List<ESubject> subjects = map.getSubjects();
        for (ESubject s : subjects) {
            try {
                N_Utils.validation(new Object[]{s.getExamid(), "考试id", 1, s.getSubjectid(), "科目id", 2});
            } catch (Exception e) {
                return new ResultVo(1, e.getMessage());
            }
        }
        eBaseService.setExam(subjects);
        return new ResultVo(0, "设置成功");
    }

    /**
     * 考试删除
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 11:30
     **/
    @PostMapping("/delete")
    public ResultVo delete(@RequestBody EBase eBase) {
        eBaseService.deleteById(eBase.getId());
        return new ResultVo(0, "删除成功");
    }

    /**
     * 考试修改
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 11:30
     **/
    @PostMapping("/update")
    public ResultVo update(@RequestBody EBase eBase) {
        return eBaseService.update(eBase);
    }

    @PostMapping("/detail")
    public ResultVo detail(@RequestBody EBase eBase) {
        Map base = eBaseService.findById(eBase.getId());
        return new ResultVo(0, "获取详情成功", base);
    }

    @PostMapping("/list")
    public ResultVo list(@RequestBody EBase eBase) {
        N_Utils.validation(new Object[]{eBase.getSchoolid(), "学校id"
                , 1, eBase.getPage(), "页码", 1, eBase.getPageSize(), "显示条数", 1});
        PageHelper.startPage(eBase.getPage(), eBase.getPageSize());
        List<Map> list = eBaseService.listAll(eBase);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo(list));
        return new ResultVo(0, "获取列表成功", pageResult);
    }

    /**
     * 发布考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 15:21
     **/
    @PostMapping("/fb")
    public ResultVo postExam(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        eBaseService.updateExamState(examid, 1);
        return new ResultVo(0, "发布成功");
    }

    /**
     * 结束考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 15:28
     **/
    @PostMapping("/js")
    public ResultVo endExam(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        eBaseService.updateExamState(examid, 2);
        return new ResultVo(0, "结束成功");
    }

    /**
     * 获取考试班级
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 16:12
     **/
    @PostMapping("/list_class")
    public ResultVo listClass(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        List<Map> list = eBaseService.listClass(examid);
        return new ResultVo(0, "获取考试班级成功", list);
    }

    /**
     * 考生管理--获取考试和班级信息
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/19 10:56
     **/
    @PostMapping("/get_eclass")
    public ResultVo getExamAndClass(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        Map map = eBaseService.getExamAndClass(examid);
        return new ResultVo(0, "获取考试班级成功", map);
    }

    /**
     * 考生管理--获取考试学生
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 16:57
     **/
    @PostMapping("/list_stu")
    public ResultVo listStu(@RequestBody EClassStudent cs) {
        Integer examid = cs.getExamid();
        Integer page = cs.getPage();
        Integer pageSize = cs.getPageSize();
        N_Utils.validation(new Object[]{examid, "考试id", 1, page, "页码", 1, pageSize, "显示条数", 1});
        PageHelper.startPage(page, pageSize);
        List<Map> list = eBaseService.listStu(cs);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0, "获取考试学生成功", pageResult);
    }

    /**
     * 考生管理--移除学生
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [student]
     * @Author ywx
     * @Date 2022/7/19 11:34
     **/
    @PostMapping("/del_stu")
    public ResultVo delStu(@RequestBody EClassStudent student) {
        Integer ecsid = student.getEcsid();
        N_Utils.validation(new Object[]{ecsid, "考试id", 1});
        eBaseService.delStu(ecsid);
        return new ResultVo(0, "移除考试学生成功");
    }

    /**
     * 考试科目列表
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/19 9:03
     **/
    @PostMapping("/list_subject")
    public ResultVo listSubject(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        Map map = eBaseService.listSubject(examid);
        return new ResultVo(0, "获取考试科目列表成功", map);
    }
}
