package com.xhkjedu.sexam.controller.exam;

import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.exam.ESubjectInvigilate;
import com.xhkjedu.sexam.service.exam.ESubjectInvigilateService;
import com.xhkjedu.sexam.vo.exam.EInVigilateVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname ESubjectInvigilateController
 * @description 监考设置
 * @date 2022/7/18 17:38
 **/
@RestController
@RequestMapping("/ei")
public class ESubjectInvigilateController {
    @Resource
    private ESubjectInvigilateService eSubjectInvigilateService;

    /**
     * 设置监考人--设置
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/19 17:32
     **/
    @PostMapping("/set")
    public ResultVo setTeacher(@RequestBody EInVigilateVo vo) {
        Integer notice = vo.getNotice();//是否发送通知1发送2不发送
        Integer msg = vo.getMsg();//是否发送短信1发送2不发送
        Integer esid = vo.getEsid();
        String subjectid = vo.getSubjectid();
        Integer createid = vo.getCreateid();
        N_Utils.validation(new Object[]{esid, "考试科目id", 1, subjectid, "科目id", 2, createid, "操作人id", 1});
        List<ESubjectInvigilate> teachers = vo.getTeachers();
        if (N_Utils.isListEmpty(teachers)) return new ResultVo(1, "监考教师不能为空");
        eSubjectInvigilateService.setTeacher(esid, subjectid, createid, teachers);
        return new ResultVo(0, "设置成功");
    }

    /**
     * 监考设置列表
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [subject]
     * @Author ywx
     * @Date 2022/7/20 17:43
     **/
    @PostMapping("/list")
    public ResultVo listSubject(@RequestBody ESubject subject) {
        Integer examid = subject.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        Map map = eSubjectInvigilateService.listSubject(examid);
        return new ResultVo(0, "获取成功", map);
    }

    /**
     * 监考人查看
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/21 9:11
     **/
    @PostMapping("/detail")
    public ResultVo detail(@RequestBody ESubject vo) {
        Integer esid = vo.getEsid();
        N_Utils.validation(new Object[]{esid, "考试科目id", 1});
        List<Map> list = eSubjectInvigilateService.listByEsId(esid);
        return new ResultVo(0, "获取成功", list);
    }

    /**
     * 监考人删除
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [vo]
     * @Author ywx
     * @Date 2022/7/21 9:17
     **/
    @PostMapping("/del")
    public ResultVo delete(@RequestBody ESubject vo) {
        Integer esid = vo.getEsid();
        N_Utils.validation(new Object[]{esid, "考试科目id", 1});
        eSubjectInvigilateService.delByEsId(esid);
        return new ResultVo(0, "删除成功");
    }
}
