package com.xhkjedu.sexam.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.*;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.sexam.vo.paper.EPaperQsVo;
import com.xhkjedu.sexam.vo.paper.EPaperQtypeQuestionVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 附件试卷
 * @Author WN
 * Date 2022/7/18 14:05
 **/
@Service
public class EPaperFjtypeService {
    @Resource
    private EPaperFjtypeMapper ePaperFjtypeMapper;
    @Resource
    private EPaperFjtypeQuestionMapper ePaperFjtypeQuestionMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperFileMapper ePaperFileMapper;
    @Resource
    private EPaperAnalyzeMapper ePaperAnalyzeMapper;

    //保存试卷
    @Transactional(rollbackFor = Exception.class)
    public void saveFjPaper(EPaper paper){
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());
        List<EPaperFjtype> fjtypelist  = paper.getFjtypelist();//题型
        List<EPaperFjtypeQuestion> queslist = new ArrayList<>();//题型下试题集合
        List<EPaperFile> files = paper.getPfiles();//附件文件
        if(files!=null && files.size()>0){
            paper.setHasfile(1);
        }else{
            paper.setHasfile(0);
        }
        setPaperBaseDetail(paper,eSubject,fjtypelist,queslist);

        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);

        if(paper.getHasfile() == 1){
            //保存作业附件
            savePaperFile(paper,files);
        }

        //保存答题卡信息
        saveFjtypeQuestion(paper,fjtypelist);

        //生成试卷分析
        savePaperAnalyze(paper,fjtypelist,queslist);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
    }

    //设置试卷信息
    private void setPaperBaseDetail(EPaper paper,ESubject eSubject,List<EPaperFjtype> fjtypelist,List<EPaperFjtypeQuestion> queslist){
        paper.setPtype(2);//试卷类型1题库2答题卡3仅答题卡
        paper.setExamid(eSubject.getExamid());
        paper.setSubjectid(eSubject.getSubjectid());
        paper.setEsid(eSubject.getEsid());
        int timestamp = N_Utils.getSecondTimestamp();

        fjtypelist.forEach(q->{
            queslist.addAll(q.getQuestions());
        });
        double pscore = queslist.stream().collect(Collectors.summingDouble(EPaperFjtypeQuestion :: getScore));
        paper.setPnum(queslist.size());
        paper.setPscore(pscore);
        paper.setCreatetime(timestamp);
        paper.setHandletime(timestamp);
        paper.setHandleid(paper.getCreateid());
        paper.setAnswered(0);
    }

    //保存附件试卷文件
    private void savePaperFile(EPaper paper,List<EPaperFile> files){
        int fileorder = 1;
        for(EPaperFile f : files){
            f.setCreateid(paper.getCreateid());
            f.setCreatetime(paper.getCreatetime());
            f.setEpid(paper.getEpid());
            f.setFileorder(fileorder);
            fileorder++;
        }
        ePaperFileMapper.insertList(files);
    }

    //保存题型--题型下试题
    private void saveFjtypeQuestion(EPaper paper,List<EPaperFjtype> fjtypelist){
        List<EPaperFjtypeQuestion> saveques = new ArrayList<>();
        for(EPaperFjtype fjt : fjtypelist){
            List<EPaperFjtypeQuestion> quelist = fjt.getQuestions();//题型下试题
            fjt.setEpid(paper.getEpid());
            fjt.setCreateid(paper.getHandleid());
            fjt.setCreatetime(paper.getHandletime());
            fjt.setEpfjnum(quelist.size());
            double tscore = quelist.stream().collect(Collectors.summingDouble(EPaperFjtypeQuestion :: getScore));
            fjt.setEpfjscore(tscore);
        }
        //批量保存题型信息
        ePaperFjtypeMapper.insertList(fjtypelist);
        for(EPaperFjtype fjt : fjtypelist){
            for(EPaperFjtypeQuestion q : fjt.getQuestions()){
                q.setEpfjid(fjt.getId());
                q.setEpid(paper.getEpid());
                saveques.add(q);
            }
        }
        ePaperFjtypeQuestionMapper.insertList(saveques);
    }

    //保存试卷分析
    private void savePaperAnalyze(EPaper ePaper,List<EPaperFjtype> fjtypelist,List<EPaperFjtypeQuestion> queslist){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        int znum = 0;
        double zscore = 0;
        int knum = 0;
        double kscore = 0;
        for(EPaperFjtypeQuestion q : queslist){
            if(N_Utils.isObjectiveQuestion(q.getCtype())){
                znum ++;
                zscore = zscore + q.getScore();
            }else{
                knum ++;
                kscore = kscore + q.getScore();
            }
        }

        double znumrate = N_Utils.getIntegerDivideAndMulitiply(znum, ePaper.getPnum());
        double knumrate = 100-znumrate;
        double zscorerate = N_Utils.getDoubleDivideAndMulitiply(zscore, ePaper.getPscore());
        double kscorerate = 100 - zscorerate;

        List<EPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new EPaperQsVo("主观题",zscore,znum,zscorerate,znumrate));
        ranglist.add(new EPaperQsVo("客观题",kscore,knum,kscorerate,knumrate));

        Map ztmap = new HashMap<>();
        ztmap.put("pnum", ePaper.getPnum());
        ztmap.put("pscore",ePaper.getPscore());
        ztmap.put("ranglist",ranglist);

        //题型分布(按照名称进行分组)
        fjtypelist.stream().sorted(Comparator.comparing(EPaperFjtype::getEpfjorder)).collect(Collectors.toList());
        Map<String,List<EPaperFjtype>> qtypemap =  fjtypelist.stream().collect(Collectors.groupingBy(EPaperFjtype:: getEpfjname, Collectors.toList()));
        List<EPaperQsVo> qtlist = new ArrayList<>();
        for(Map.Entry<String, List<EPaperFjtype>> entry : qtypemap.entrySet()){
            List<EPaperFjtype> aq = entry.getValue();
            int num = aq.stream().mapToInt(EPaperFjtype::getEpfjnum).sum();
            double tscore = aq.stream().collect(Collectors.summingDouble(EPaperFjtype:: getEpfjscore));
            double storerate = N_Utils.getDoubleDivideAndMulitiply(tscore, ePaper.getPscore());
            EPaperQsVo qvo = new EPaperQsVo();
            qvo.setQtname(entry.getKey());
            qvo.setNum(num);
            qvo.setScore(tscore);
            qvo.setSrate(storerate);
            qtlist.add(qvo);
        }

        EPaperAnalyze epa = new EPaperAnalyze();
        epa.setEpaid(ePaper.getEpid());
        epa.setAlljson(JSON.toJSONString(ztmap));
        epa.setQnumjson(JSON.toJSONString(qtlist));
        ePaperAnalyzeMapper.insertUseGeneratedKeys(epa);
    }

    @Transactional(rollbackFor = Exception.class)
    public void updateFjPaper(EPaper paper){
        //先删除原考试关联的试卷信息
        ePaperMapper.deletePaperByEpid(paper.getEpid());
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());
        List<EPaperFjtype> fjtypelist  = paper.getFjtypelist();//题型
        List<EPaperFjtypeQuestion> queslist = new ArrayList<>();//题型下试题集合
        List<EPaperFile> files = paper.getPfiles();//附件文件
        if(files!=null && files.size()>0){
            paper.setHasfile(1);
        }else{
            paper.setHasfile(0);
        }
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        setPaperBaseDetail(paper,eSubject,fjtypelist,queslist);
        if(paper.getHasfile() == 1){
            //保存作业附件
            savePaperFile(paper,files);
        }
        //保存答题卡信息
        saveFjtypeQuestion(paper,fjtypelist);

        //生成试卷分析
        savePaperAnalyze(paper,fjtypelist,queslist);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
    }

    /**
     * @Description 试卷详情
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/19 8:35
     **/
    public Map getPaperFjDetail(Integer epid){
        Map map = ePaperMapper.getExamPaperDetailByEpid(epid);//试卷信息
        List<Map> fjtypelist = ePaperFjtypeMapper.listPaperFjtypeQuestions(epid);//试卷题型试题信息
        int hasfile = Integer.parseInt(map.get("hasfile").toString());
        List<Map> fileslist = new ArrayList<>();
        if(hasfile == 1){
            fileslist = ePaperFileMapper.listPaperFiles(epid);//附件集合
        }

        map.put("qtypes",fjtypelist);
        map.put("files",fileslist);
        return map;
    }

    /**
     * @Description 保存试卷设置正确答案
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/19 15:00
     **/
    public void savePaperFjQuestionAnswer(EPaper paper){
        List<EPaperFjtypeQuestion> ques = paper.getFjqueslist();
        //更新试卷试题答案
        List<EPaperFjtypeQuestion> updqu = new ArrayList<>();
        for(EPaperFjtypeQuestion q: ques){
            if(N_Utils.isNotEmpty(q.getAnswer())){
                List<String> anslist = JSON.parseArray(q.getAnswer(),String.class);
                if(anslist!=null && anslist.size()>0){
                    updqu.add(q);
                }
            }
        }
        //更改试题答案
        ePaperFjtypeQuestionMapper.updateBatchFjQuestionAnswer(updqu);
        //更改试卷是否设置设置答案
        paper.setAnswered(1);
        ePaperMapper.updateEPaperAnswered(paper);
        //自动批改学生客观题批阅---------------------------------------------
    }
}
