package com.xhkjedu.sexam.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperAnalyzeMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperQtypeMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperQtypeQuestionMapper;
import com.xhkjedu.sexam.mapper.system.UserBasketMapper;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.sexam.vo.paper.EPaperQPointVo;
import com.xhkjedu.sexam.vo.paper.EPaperQsVo;
import com.xhkjedu.sexam.vo.paper.EPaperQtypeQuestionVo;
import com.xhkjedu.sexam.vo.paper.EPaperQtypeVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 题库题型
 * @Author WN
 * Date 2022/7/19 9:07
 **/
@Service
public class EPaperQtypeService {
    @Resource
    private EPaperQtypeMapper ePaperQtypeMapper;
    @Resource
    private EPaperQtypeQuestionMapper ePaperQtypeQuestionMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperAnalyzeMapper ePaperAnalyzeMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;

    @Resource
    private UserBasketMapper userBasketMapper;


    /**
     * @Description 保存题库试卷
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/19 9:10
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveQtPaper(EPaper paper) {
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());
        List<EPaperQtype> qtypelist = paper.getQtypelist();
        List<EPaperQtypeQuestion> queslist = new ArrayList<>();//存放试题
        setEPaperBaseDetail(paper,eSubject,qtypelist,queslist);
        //保存试卷信息
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        //保存题型试题信息
        savePaperQtype(paper,qtypelist);
        //保存作业分析
        savePaperAnalyze(paper);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
        //保存后清除试题栏中试题
        userBasketMapper.deleteByUseridAndSubjectid(paper.getCreateid(),eSubject.getSubjectid());
    }

    //设置试卷基本信息
    private void setEPaperBaseDetail(EPaper paper,ESubject eSubject,List<EPaperQtype> qtypelist,List<EPaperQtypeQuestion> queslist){
        qtypelist.forEach(q ->{
            queslist.addAll(q.getQuestions());
        });
        paper.setPtype(1);//试卷类型1题库2答题卡3仅答题卡
        paper.setExamid(eSubject.getExamid());
        paper.setSubjectid(eSubject.getSubjectid());
        paper.setEsid(eSubject.getEsid());
        int timestamp = N_Utils.getSecondTimestamp();
        double pscore = queslist.stream().collect(Collectors.summingDouble(EPaperQtypeQuestion:: getScore));
        paper.setPnum(queslist.size());
        paper.setPscore(pscore);
        paper.setCreatetime(timestamp);
        paper.setHandletime(timestamp);
        paper.setHandleid(paper.getCreateid());
        paper.setAnswered(1);
    }

    //保存试卷题型试题信息
    private void savePaperQtype(EPaper epaper,List<EPaperQtype> qtypelist){
        List<EPaperQtypeQuestion> qtqs = new ArrayList<>();
        for(EPaperQtype qtype : qtypelist){
            List<EPaperQtypeQuestion> qlst = qtype.getQuestions();
            qtype.setEpid(epaper.getEpid());
            qtype.setCreateid(epaper.getHandleid());
            qtype.setCreatetime(epaper.getHandletime());
            qtype.setEptnum(qlst.size());
            Double ptscore = qlst.stream().collect(Collectors.summingDouble(EPaperQtypeQuestion::getScore));
            qtype.setEptscore(ptscore);
        }
        ePaperQtypeMapper.insertList(qtypelist);//保存题型
        for(EPaperQtype qtype : qtypelist){
            for(EPaperQtypeQuestion q : qtype.getQuestions()){
                q.setEpid(epaper.getEpid());
                q.setEptid(qtype.getId());
                qtqs.add(q);
            }
        }
        ePaperQtypeQuestionMapper.insertList(qtqs);
    }

    //保存试卷分析List<EPaperQtype> qtypelist,List<EPaperQtypeQuestion> queslist
    private void savePaperAnalyze(EPaper ePaper){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        List<EPaperQtypeVo> qtypelist = ePaperQtypeMapper.listPaperQtypeQuestions(ePaper.getEpid());//试卷题型试题信息
        List<EPaperQtypeQuestionVo> queslist = new ArrayList<>();//试卷中所有试题(子题)
        List<EPaperQPointVo> quespoints = new ArrayList<>();//试卷中所有单题和子题以及对应知识点

        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        List<EPaperQsVo> typelist = new ArrayList<>();//题型分析
        //处理出试卷中所有试题、知识点
        for(EPaperQtypeVo qt : qtypelist){
            for(EPaperQtypeQuestionVo q : qt.getQuestions()){
                //处理试题关联知识点
                List<EPaperQPointVo> pointslist = q.getPoints();
                if(pointslist != null && pointslist.size()>0){
                    for(EPaperQPointVo point : pointslist){
                        EPaperQPointVo p = new EPaperQPointVo();
                        p.setPointid(point.getPointid());
                        p.setPointname(point.getPointname());
                        p.setQuestionid(q.getQuestionid());
                        p.setOrder(q.getOrder());
                        p.setScore(q.getScore());
                        quespoints.add(p);
                    }
                }
                if(q.getQlevel()==1){
                    q.setQuestionpid("000");
                }
                queslist.add(q);
            }

            double storerate = N_Utils.getDoubleDivideAndMulitiply(qt.getEptscore(), ePaper.getPscore());
            EPaperQsVo tvo = new EPaperQsVo();
            tvo.setQtname(qt.getEptname());
            tvo.setNum(qt.getEptnum());
            tvo.setScore(qt.getEptscore());
            tvo.setSrate(storerate);
            typelist.add(tvo);
        }

        //获取出所有单题和母题,进行判断区分
        Map<String,List<EPaperQtypeQuestionVo>> quesMap = queslist.stream().collect(Collectors.groupingBy(EPaperQtypeQuestionVo:: getQuestionpid, Collectors.toList()));
        List<EPaperQtypeQuestionVo> mquestions = new ArrayList<>();//仅存放单题和母题
        for(Map.Entry<String, List<EPaperQtypeQuestionVo>> entry : quesMap.entrySet()){
            if(entry.getKey().equals("000")){
                for(EPaperQtypeQuestionVo dq : entry.getValue()){
                    if(N_Utils.isObjectiveQuestion(dq.getCtype())){
                        //判断试题是否是客观题,暂时用mctype进行标识
                        dq.setMctype(111);
                    }else{
                        dq.setMctype(112);
                    }
                    mquestions.add(dq);
                }
            }else{
                EPaperQtypeQuestionVo mq = new EPaperQtypeQuestionVo();
                List<EPaperQtypeQuestionVo> sonques = entry.getValue();
                int k = 0;//如果k大于1说明有小题是主观题，则母题为主观题
                double score = 0;
                for(EPaperQtypeQuestionVo sq : sonques){
                    if(!N_Utils.isObjectiveQuestion(sq.getCtype())){
                        k++;
                    }
                    score = N_Utils.getDoubleSum(score,sq.getScore());
                }

                EPaperQtypeQuestionVo qobj = sonques.get(0);
                mq.setQuestionid(qobj.getQuestionpid());
                mq.setComplexity(qobj.getMcomplexity());
                mq.setScore(score);
                if(k == 0){
                    mq.setMctype(111);
                }else{
                    mq.setMctype(112);
                }
                mquestions.add(mq);
            }
        }

        //试题总体分布进行分析
        //客观题集合
        List<EPaperQtypeQuestionVo> ktlst = mquestions.stream().filter(q->q.getMctype() == 111).collect(Collectors.toList());
        int ktnum = ktlst.size();
        double ktscore = ktlst.stream().collect(Collectors.summingDouble(EPaperQtypeQuestionVo:: getScore));
        double ktnrate = N_Utils.getIntegerDivideAndMulitiply(ktnum, ePaper.getPnum());
        double ktsrate = N_Utils.getDoubleDivideAndMulitiply(ktscore, ePaper.getPscore());
        List<EPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new EPaperQsVo("客观题",ktscore,ktnum,ktsrate,ktnrate));
        EPaperQsVo zgq = new EPaperQsVo();
        zgq.setQtname("主观题");
        zgq.setNum(ePaper.getPnum() - ktnum);
        zgq.setNrate(N_Utils.getDoubleReduce(100.0,ktnrate));
        zgq.setScore(N_Utils.getDoubleReduce(ePaper.getPscore(),ktscore));
        zgq.setSrate(N_Utils.getDoubleReduce(100.0,ktsrate));
        ranglist.add(zgq);
        Map ztmap = new HashMap<>();
        ztmap.put("pnum", ePaper.getPnum());
        ztmap.put("pscore",ePaper.getPscore());
        ztmap.put("ranglist",ranglist);

        //难易度
        Map<Integer,List<EPaperQtypeQuestionVo>> complxmap =  mquestions.stream().collect(Collectors.groupingBy(EPaperQtypeQuestionVo:: getComplexity, Collectors.toList()));
        List<Map> clist = new ArrayList<>();
        for(Map.Entry<Integer, List<EPaperQtypeQuestionVo>> entry : complxmap.entrySet()){
            Map cmap = new HashMap();
            cmap.put("level",entry.getKey());
            List<EPaperQtypeQuestionVo> aq = entry.getValue();
            double cscore = aq.stream().collect(Collectors.summingDouble(EPaperQtypeQuestionVo:: getScore));
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(cscore, ePaper.getPscore());
            cmap.put("score",cscore);
            cmap.put("srate",scorerate);
            clist.add(cmap);
        }

        //知识点
        Map<String,List<EPaperQPointVo>> pointmap =  quespoints.stream().collect(Collectors.groupingBy(EPaperQPointVo:: getPointid, Collectors.toList()));
        List<Map> pointlist = new ArrayList<>();
        for(Map.Entry<String, List<EPaperQPointVo>> entry : pointmap.entrySet()){
            Map pm = new HashMap();
            List<EPaperQPointVo> pvo = entry.getValue();
            double cscore = pvo.stream().collect(Collectors.summingDouble(EPaperQPointVo:: getScore));
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(cscore, ePaper.getPscore());
            List<Integer> thlist = pvo.stream().map(EPaperQPointVo :: getOrder).collect(Collectors.toList());
            pm.put("pointname",pvo.get(0).getPointname());
            pm.put("score",cscore);
            pm.put("srate",scorerate);
            pm.put("order",thlist.stream().distinct().collect(Collectors.toList()));
            pointlist.add(pm);
        }

        EPaperAnalyze epa = new EPaperAnalyze();
        epa.setEpid(ePaper.getEpid());
        epa.setAlljson(JSON.toJSONString(ztmap));//总体分析
        epa.setQnumjson(JSON.toJSONString(typelist));//试题栏题型分析
        epa.setComplexityjson(JSON.toJSONString(clist));//难易度分析
        epa.setPointjson(JSON.toJSONString(pointlist));//知识点分析
        ePaperAnalyzeMapper.insertUseGeneratedKeys(epa);

    }

    /**
     * @Description 修改试卷试题
     * @Param [paper]
     * @Return void
     * @Author wn
     * @Date 2022/7/20 8:43
     **/
    @Transactional(rollbackFor = Exception.class)
    public void updateQtPaper(EPaper paper) {
        //删除原试卷
        ePaperMapper.deletePaperByEpid(paper.getEpid());
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());
        List<EPaperQtype> qtypelist = paper.getQtypelist();
        List<EPaperQtypeQuestion> queslist = new ArrayList<>();//存放试题
        setEPaperBaseDetail(paper,eSubject,qtypelist,queslist);
        //保存试卷信息
        ePaperMapper.insertUseGeneratedKeys(paper);
        int epid = paper.getId();
        paper.setEpid(epid);
        //保存题型试题信息
        savePaperQtype(paper,qtypelist);
        //保存作业分析
        savePaperAnalyze(paper);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，更改科目状态
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
    }

    /**
     * @Description  题库试卷详情
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/19 9:10
     **/
    public Map getPaperQtDetail(Integer epid) {
        Map map = ePaperMapper.getExamPaperDetailByEpid(epid);//试卷信息
        List<EPaperQtypeVo> fjtypelist = ePaperQtypeMapper.listPaperQtypeQuestions(epid);//试卷题型试题信息
        map.put("qtypes",fjtypelist);
        return map;
    }
}
