package com.xhkjedu.sexam.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperFjtypeMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperQtypeMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperScantronMapper;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.IntStream;

@Service
public class EPaperScantronService {
    @Resource
    private EPaperScantronMapper ePaperScantronMapper;
    @Resource
    private EPaperQtypeMapper ePaperQtypeMapper;
    @Resource
    private EPaperFjtypeMapper ePaperFjtypeMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;

    /**
     * @Description 获取制作答题卡需要的试卷信息
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/22 10:19
     **/
    public Map getPaperAndQuesitons(Integer epid){
        Map eps = ePaperMapper.getExamPaperAndSubject(epid);//考试考试科目名称
        List<Map> qtypes = new ArrayList<>();//题型试题
        int ptype = Integer.parseInt(eps.get("ptype").toString());//试卷类型1题库2附件
        if(ptype == 1){
            qtypes = ePaperQtypeMapper.listPaperqtypeQuestionsForScan(epid);
            //处理试卷中试题选项数量
            for(Map qtm : qtypes){
                List<Map> qlist = (List<Map>) qtm.get("questions");
                for(Map qm : qlist){
                    String qoption = qm.get("qoption").toString();
                    Integer ctype = Integer.parseInt(qm.get("ctype").toString());
                    if(ctype == 1 || ctype ==2 || ctype==4 || ctype ==5 || ctype==6 ){
                        List<String> strlst = JSON.parseArray(qoption,String.class);
                        qm.put("qoption","");
                        qm.put("optionnum",strlst.size());
                    }
                }
            }
        }else{
            qtypes = ePaperFjtypeMapper.listPaperfjtypeQuestionsForScan(epid);
        }
        Map rtnmap = new HashMap();
        rtnmap.put("paper",eps);
        rtnmap.put("qtypes",qtypes);
        return rtnmap;
    }

    /**
     * @Description 保存答题卡
     * @Param [ePaper]
     * @Return void
     * @Author wn
     * @Date 2022/7/22 14:16
     **/
    @Transactional(rollbackFor = Exception.class)
    public void savePaperScantron(EPaper ePaper) {
        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(ePaper.getEsid());
        ePaper.setEsid(eSubject.getEsid());
        List<EPaperScantron> sctypelist  = ePaper.getScanlist();//答题卡
        int[] ctypes = new int[]{3,11,13,14,15};//ctype为指定值时会出现答题卡不在一个区域
        int timestamp = N_Utils.getSecondTimestamp();
        for(EPaperScantron ct : sctypelist){
            ct.setCreateid(ePaper.getCreateid());
            ct.setCreatetime(timestamp);
            if(IntStream.of(ctypes).anyMatch(x -> x == ct.getCtype())){
                List<List> savecoord = new ArrayList<>();
                for(String str : ct.getCoordlist()){
                    List<List> strlist = JSON.parseArray(str, List.class);
                    for(List l : strlist){
                        savecoord.add(l);
                    }
                }
                ct.setCoord(JSON.toJSONString(savecoord));
            }else{
                ct.setCoord(ct.getCoordlist().get(0));
            }
        }
        //先删除试题原有答题卡
        ePaperScantronMapper.deleteScantronByEpid(ePaper.getId());
        //批量保存答题卡信息
        ePaperScantronMapper.insertList(sctypelist);
        //完善试卷信息
        ePaperMapper.updatePaperScantron(ePaper);
    }



}
